//GAMY: a tool for estimating the similarity of graphs
//	Copyright(C) 2019 Mattia De Prisco and Giovanni Squillero
//
//	This program is free software : you can redistribute it and /or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.If not, see < http://www.gnu.org/licenses/>.

#include "argsreader.h"
#include "montecarlonode.h"
#include "montecarlotree.h"
#include <iostream>

std::vector<Graph> ArgsReader::getArguments(int argc, char** argv) {
    std::vector<std::string> args(argv + sizeof(char), argv + argc);
	std::vector<Graph> graph_pack;
	for(size_t i = 0; i < args.size(); i += 2) {
		std::string arg = args.at(i);
		if(arg == "-g")
			graph_pack.emplace_back(args.at(i+1));
		else if(arg == "-t")
			MonteCarloTree::TIMEOUT = std::stoi(args.at(i + 1));
		else if(arg == "-em")
			PartitionManager::EQUIVALENCE_MODIFIER = std::stod(args.at(i + 1));
		else if(arg == "-hr")
			PartitionManager::HOPE_REDUCER = std::stod(args.at(i + 1));
		else if(arg == "-nhm")
			MonteCarloNode::NODE_HOPE_MODIFIER = std::stod(args.at(i + 1));
		else if(arg == "-fm")
			MonteCarloNode::FACT_MODIFIER = std::stod(args.at(i + 1));
		else if (arg == "-bwr")
			MonteCarloTree::BAD_WEIGHT_RATIO = std::stod(args.at(i + 1));
		else if(arg == "-piv")
			MonteCarloNode::PROPAGATION_INITIAL_VALUE = std::stod(args.at(i + 1));
		else if(arg == "-pr")
			MonteCarloNode::PROPAGATION_MODIFIER = std::stod(args.at(i + 1));
		else if (arg == "-mc")
			MonteCarloTree::MIN_CANDIDATES = std::stoi(args.at(i + 1));
		else if (arg == "-l")
			Graph::VERTEX_LABELS_ON = std::stoi(args.at(i + 1));
		else if (arg == "-r")
			MonteCarloTree::RESULT_FOLDER = std::string(args.at(i + 1));
	}

	return graph_pack;
}

void ArgsReader::printUsage(char** argv) {
	std::cout << "Usage: " << std::endl;
	std::cout << argv[0] << " -g path_to_graph_0 -g path_to_graph_1 [... -g path_to_graph_N] [-t A] [-em B] [-hr C] [-nhm D] [-fm E] [-piv F] [-pr G] [-mc H] [-l I] [-r L]" << std::endl;
	std::cout << "Graphs format supported: .mat, .dimacs, .ladder" << std::endl;
	std::cout << "Constraints: " << std::endl;
	std::cout << "-g graphPath	-> min 2 graphs (formats supported: mat, dimacs, ladder, yara)" << std::endl;
	std::cout << "-t A			-> A >= 0		(uint),		default = 10 - timeout in seconds, set to 0 for endless mode" << std::endl;
	std::cout << "-em B			-> B >= 1		(double),	default = 2.0" << std::endl;
	std::cout << "-hr C			-> 0 < C < 1	(double),	default = 0.8" << std::endl;
	std::cout << "-nhm D		-> D > 0		(double),	default = 1.0" << std::endl;
	std::cout << "-fm E			-> E > 0		(double),	default = 10.0" << std::endl;
	std::cout << "-bwr F		-> F > 0		(double),	default = 1.0" << std::endl;
	std::cout << "-piv G		-> 0 < G < 1	(double),	default = 1.0" << std::endl;
	std::cout << "-pr H			-> 0 < H < 1	(double),	default = 0.8" << std::endl;
	std::cout << "-mc I			-> I >= 1		(uint),		default = 1" << std::endl;
	std::cout << "-l J			-> J == 0 | 1	(uint),		default = 1 - 1 means labels activated on vertexes, 0 means no labels" << std::endl;
	std::cout << "-r K			-> K == \"path_to_result_folder\" (string), default = \"../results\"" << std::endl;

}