//GAMY: a tool for estimating the similarity of graphs
//	Copyright(C) 2019 Mattia De Prisco and Giovanni Squillero
//
//	This program is free software : you can redistribute it and /or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.If not, see < http://www.gnu.org/licenses/>.

#pragma once

#include "graph.h"
#include <vector>
#include <set>

class History
{
	struct HistoryComparator {
		bool operator() (std::vector<node_t> lhs, std::vector<node_t> rhs) const {
			if (lhs.size() == rhs.size()) {
				return (lhs)[0] < (rhs)[0];
			}
			return lhs.size() < rhs.size();
		}
	};

	std::set<std::vector<node_t>, HistoryComparator> choices_;

public:
	History() = default;
	~History() = default;

	History(const History& other) =	default;;
	History& operator=(const History& other) = default;
	
	History(History&& other) noexcept : choices_(std::move(other.choices_)) {};
	History& operator=(History&& other) noexcept {
		choices_ = std::move(other.choices_);
		return *this;
	};

	bool operator==(History const& other) const {
		return choices_ == other.choices_;
	}

	void addChoice(const std::vector<node_t>& new_choice) { choices_.insert(new_choice); }
	const std::set<std::vector<node_t>, HistoryComparator>& getChoices() const { return choices_; }
};

namespace std {
	template<> struct hash<History> {
		size_t operator()(const History& t) const noexcept {

			std::size_t seed = t.getChoices().size();
			for(const auto& choice : t.getChoices()) {
				for(const auto& n : choice) {
					seed ^= n + 0x9e3779b9 + (seed << 6) + (seed >> 2);
				}
			}
			return seed;
		}
	};
}
