//GAMY: a tool for estimating the similarity of graphs
//	Copyright(C) 2019 Mattia De Prisco and Giovanni Squillero
//
//	This program is free software : you can redistribute it and /or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.If not, see < http://www.gnu.org/licenses/>.

/*-*
 *  It doesn't matter what you create
 *  If you have no fun
 *          -- Michelle Gurevich, "Party Girl"
 *-*/

#include "graph.h"
#include "montecarlonode.h"
#include <vector>
#include "selector.h"
#include "argsreader.h"
#include "montecarlotree.h"
#include <iostream>

int main(int argc, char** argv) {
    // read graphs and parameters from argv & build initial partitions
	std::vector<Graph> graph_pack;
	try {
		graph_pack = ArgsReader::getArguments(argc, argv);
		if (graph_pack.size() <= 1) {
			throw GraphException("At least two graphs are required.");;
		}
	} catch (GraphException& e) {
		std::cout << e.good_reason << std::endl;
		ArgsReader::printUsage(argv);
		return 1;
	}

    std::vector<std::vector<node_t>> initial_partitions;
	
	size_t n_initial_partitions = 0;
	for (const auto& graph : graph_pack)
		n_initial_partitions += graph.size();

	initial_partitions.reserve(n_initial_partitions);

	for(const auto& graph : graph_pack)
		initial_partitions.push_back(graph.nodeList());
    
	PartitionManager hc{std::move(initial_partitions)};
	hc.sort(PartitionManager::SortStrategy::DEGREE);

    // update partitions using labels
    std::unordered_set<std::string> labels;
    for(const auto& g : graph_pack) {
        for(const auto& n : g.getNodes()) {
            labels.insert(n->label);
        }
    }
    for(const auto& l : labels) {
        std::vector<node_t> c;
        for(int i = 0; i < static_cast<int>(NodeStore::size()); ++i) {
            if(NodeStore::getNode(i)->label == l)
                c.push_back(i);
        }
        hc.split(c);
    }

    // run montercarlo tree search
	MonteCarloTree::MonteCarloTreeSearch(std::move(hc), std::move(graph_pack));
    return 0;
	
}

