//GAMY: a tool for estimating the similarity of graphs
//	Copyright(C) 2019 Mattia De Prisco and Giovanni Squillero
//
//	This program is free software : you can redistribute it and /or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.If not, see < http://www.gnu.org/licenses/>.

#pragma once

#include "montecarlonode.h"

class MonteCarloNode;

class NodeSelector {
	std::vector<std::weak_ptr<MonteCarloNode>> entries_;
	double total_;

public:

	NodeSelector() :total_(0) {};

	size_t size() const { return entries_.size(); }
	std::pair<std::weak_ptr<MonteCarloNode>, size_t> getWorstCandidate();
	void clear() { entries_.clear(); }
	std::pair<std::weak_ptr<MonteCarloNode>, size_t > getCandidate();
	bool contains(std::weak_ptr<MonteCarloNode>& node_to_find);
	void addEntry(std::weak_ptr<MonteCarloNode>&& entry);
	void rmvEntry(const size_t pos);
	void sortByWeight();
	void updateTotal(const double value) { total_ += value; };
	double getNormValue(const size_t pos) const;
	std::pair<std::weak_ptr<MonteCarloNode>, size_t > rouletteWheel() const;
	bool isSorted();
};
